SystemOrganization addCategory: #'Comet-Examples'!

!WAPackage class methodsFor: '*comet-examples' stamp: 'lr 3/16/2009 20:08'!
cometExamples
	^ self new
		name: 'Comet-Examples';
		description: 'Examples for server push technology.';
		addDependency: 'JQuery-Core';
		addDependency: 'Comet-Core';
		yourself! !

WAComponent subclass: #CTChat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Examples'!

CTChat class
	instanceVariableNames: 'pusher'!

CTChat class
	instanceVariableNames: 'pusher'!

!CTChat class methodsFor: 'initialization' stamp: 'lr 3/16/2009 19:51'!
initialize
	| application |
	application := self registerAsApplication: 'comet/chat'.
	application preferenceAt: #sessionClass put: CTChatSession.
	application addLibrary: JQDeploymentLibrary; addLibrary: CTLibrary! !

!CTChat class methodsFor: 'accessing' stamp: 'lr 1/2/2008 17:58'!
pusher
	^ pusher ifNil: [ pusher := CTPusher new ]! !

!CTChat methodsFor: 'testing' stamp: 'lr 1/2/2008 19:00'!
isLoggedIn
	^ self username notNil! !

!CTChat methodsFor: 'actions' stamp: 'lr 1/2/2008 20:01'!
login: aString
	"Join the chat-room with the given username aString."
	
	aString isEmptyOrNil ifTrue: [ ^ self ].
	self session username: aString.
	self push: 'event' with: 'joined'! !

!CTChat methodsFor: 'actions' stamp: 'lr 1/2/2008 20:01'!
logout
	"Leave the chat-room."

	self push: 'event' with: 'left'.
	self session username: nil! !

!CTChat methodsFor: 'actions' stamp: 'lr 1/2/2008 19:35'!
message: aString
	"Send the message aString to the chat-room."
	
	self push: 'message' with: aString! !

!CTChat methodsFor: 'private' stamp: 'lr 3/16/2009 19:54'!
push: aString with: aBlock
	self pusher javascript: [ :script |
		script << (script jQuery: #messages)
			prepend: [ :html | self renderMessage: aBlock class: aString on: html ] ]! !

!CTChat methodsFor: 'accessing' stamp: 'lr 1/2/2008 18:37'!
pusher
	^ self class pusher! !

!CTChat methodsFor: 'rendering-chat' stamp: 'lr 3/17/2009 20:22'!
renderChatInputOn: html
	html textArea
		id: #input;
		callback: [ :value | self message: value ];
		onEnter: (html jQuery ajax
			onSuccess: ((html jQuery: #input) 
				value: String new);
			serializeThis)! !

!CTChat methodsFor: 'rendering-chat' stamp: 'lr 1/2/2008 20:05'!
renderChatMessagesOn: html
	html div id: 'messages'; with: [
		self 
			renderMessage: [
				self users isEmpty
					ifTrue: [ html text: 'You are the only one here!!' ]
					ifFalse: [
						html text: 'Also there: '.
						self users
							do: [ :each | html text: each ]
							separatedBy: [ html text: ', ' ] ] ]
			class: 'event' on: html ]! !

!CTChat methodsFor: 'rendering' stamp: 'lr 1/2/2008 19:44'!
renderChatOn: html
	self renderChatStatusOn: html.
	self renderChatMessagesOn: html.
	self renderChatInputOn: html.
	self renderChatPusherOn: html! !

!CTChat methodsFor: 'rendering-chat' stamp: 'lr 3/25/2009 08:56'!
renderChatPusherOn: html
	html document 
		addLoadScript: (html comet
			pusher: self pusher;
			connect)! !

!CTChat methodsFor: 'rendering-chat' stamp: 'lr 1/2/2008 19:56'!
renderChatStatusOn: html
	html div id: 'status'; with: [
		html text: 'Currently logged in as ' , self username , '. '.
		html anchor
			callback: [ self logout ];
			with: 'Logout' ]! !

!CTChat methodsFor: 'rendering' stamp: 'lr 1/2/2008 18:38'!
renderContentOn: html
	html heading: 'Seaside Chat'.
	self isLoggedIn
		ifTrue: [ self renderChatOn: html ]
		ifFalse: [ self renderLoginOn: html ]! !

!CTChat methodsFor: 'rendering' stamp: 'lr 1/2/2008 18:52'!
renderLoginOn: html
	html form: [
		html textInput
			value: self username;
			callback: [ :value | self login: value ].
		html submitButton
			text: 'Login' ]! !

!CTChat methodsFor: 'rendering' stamp: 'lr 1/2/2008 20:05'!
renderMessage: aBlock class: aString on: html
	html div class: 'message'; with: [
		html span class: 'user'; with: self username.
		html span class: aString; with: aBlock ]! !

!CTChat methodsFor: 'rendering' stamp: 'lr 1/10/2009 13:36'!
style
	^ 'a {
	color: #00f;
	cursor: pointer;
	text-decoration: underline;	
}
#status {
	background-color: #eee;
}
#messages {
	height: 10em;
	overflow: auto;
}
#messages .message {
	margin-top: 0.25em;
	margin-bottom: 0.25em;
}
#messages .message .user {
	font-weight: bold;
	margin-right: 1em;
}
#messages .message .event {
	font-style: italic;
}
#input {
	width: 100%;
	background-color: #eee;
}'! !

!CTChat methodsFor: 'accessing' stamp: 'lr 1/2/2008 18:38'!
username
	^ self session username! !

!CTChat methodsFor: 'accessing' stamp: 'lr 1/2/2008 20:01'!
users
	^ (self pusher handlers
		reject: [ :each | self session == each session or: [ each session username isNil ] ])
		collect: [ :each | each session username ]! !

WAComponent subclass: #CTCounter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Examples'!

CTCounter class
	instanceVariableNames: 'model pusher'!

CTCounter class
	instanceVariableNames: 'model pusher'!

!CTCounter class methodsFor: 'initialization' stamp: 'lr 3/16/2009 19:30'!
initialize
	| application |
	application := WAAdmin 	register: self asApplicationAt: 'comet/counter'.
	application addLibrary: JQDeploymentLibrary; addLibrary: CTLibrary! !

!CTCounter class methodsFor: 'accessing' stamp: 'lr 1/2/2008 18:01'!
model
	^ model ifNil: [ model := CTCounterModel new ]! !

!CTCounter class methodsFor: 'accessing' stamp: 'lr 1/2/2008 18:01'!
pusher
	^ pusher ifNil: [ pusher := CTPusher new ]! !

!CTCounter methodsFor: 'actions' stamp: 'lr 1/2/2008 17:08'!
decrease
	self model decrease.
	self update: #decrease! !

!CTCounter methodsFor: 'actions' stamp: 'lr 1/2/2008 17:08'!
increase
	self model increase.
	self update: #increase! !

!CTCounter methodsFor: 'accessing' stamp: 'lr 1/2/2008 16:38'!
model
	^ self class model! !

!CTCounter methodsFor: 'accessing' stamp: 'lr 1/2/2008 16:38'!
pusher
	^ self class pusher! !

!CTCounter methodsFor: 'rendering' stamp: 'lr 3/25/2009 08:56'!
renderContentOn: html
	html heading
		id: #count;
		with: self model count.
	html paragraph: [
		html anchor
			onClick: (html jQuery ajax
				callback: [ self decrease ]);
			with: 'dec'.
		html space; space; space.
		html anchor
			onClick: (html jQuery ajax
				callback: [ self increase ]);
			with: 'inc' ].
	html document 
		addLoadScript: (html comet
			pusher: self pusher;
			connect)! !

!CTCounter methodsFor: 'rendering' stamp: 'lr 3/16/2009 19:50'!
style
	^ 'body {
	color: #222;
	font-size: 4em;
	line-height: 1.5;
	text-align: center;
	font-family: Helvetica, Arial, sans-serif;
}
h1 { font-weight: bold; }
a {
	color: #555;
	cursor: pointer;
	text-decoration: none;	
}'! !

!CTCounter methodsFor: 'private' stamp: 'lr 3/16/2009 19:38'!
update: anAspect
	"Update the number in all connected web browser. Only the first 4 lines are really needed, the rest adds some eye-candy and can simply be removed if not needed."

	self pusher javascript: [ :script |
		script << (script jQuery: #count) 
			text: self model count;
			append: [ :html |
				html span: [
					html html: (anAspect = #increase
						ifTrue: [ '&uarr;' ]
						ifFalse: [ '&darr;' ]) ] ];
			find: 'span'; fadeOut: 'slow' ]! !

WASession subclass: #CTChatSession
	instanceVariableNames: 'username'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Examples'!

!CTChatSession methodsFor: 'accessing' stamp: 'lr 1/2/2008 18:07'!
username
	^ username! !

!CTChatSession methodsFor: 'accessing' stamp: 'lr 1/2/2008 18:06'!
username: aString
	username := aString! !

WAPlatformObject subclass: #CTCounterModel
	instanceVariableNames: 'count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Examples'!

!CTCounterModel methodsFor: 'accessing' stamp: 'lr 1/2/2008 16:36'!
count
	^ count! !

!CTCounterModel methodsFor: 'accessing' stamp: 'lr 1/2/2008 16:43'!
count: anInteger
	count := anInteger! !

!CTCounterModel methodsFor: 'actions' stamp: 'lr 1/2/2008 16:33'!
decrease
	count := count - 1! !

!CTCounterModel methodsFor: 'actions' stamp: 'lr 1/2/2008 16:33'!
increase
	count := count + 1! !

!CTCounterModel methodsFor: 'initialization' stamp: 'lr 11/11/2008 22:57'!
initialize
	super initialize.
	count := 0! !
CTChat initialize!
CTCounter initialize!
